/*************************************************************************************************************************
									File Details
**************************************************************************************************************************
	File name 							: SystemInit.C
	
	Purpose								: This file has driver functions to initialize processor specific modules such as
									      PLL, SRU, DDR2/SDRAM, AMI controller  etc. These functions are called single time 
									      at the begining of the code.
	
	Author								: Mitesh Moonat & Harshit Gaharwar
	
	VDSP version used for testing		: VisualDSP++ 5.0, Update 10
	
	Hardware used						: ADSP-21469, ADSP-21479, ADSP-21489, ADSP-21375, ADSP-21369, ADSP-21364,
										  ADSP-21262  EZ-Kits
	
	Connection details 					: See "main.C"
	
	Guidelines for change				: 1) Following changes may be required for different processors or FLASH device
											
										  - The CCLK, SDCLK/DDR2CLK, SRU settings are currently set similar to the 
										  	EZ-Kit examples.They can/should be changed if required for custom board. 
										  	One should change the variables g_fPCLK accordingly.
										  	
										  - No changes required to this file if the code has to run on any of the EZ-Kits
**************************************************************************************************************************/

/*************************************************************************************************************************
									Include Files
*************************************************************************************************************************/
#include "SystemInit.h"



/*************************************************************************************************************************
									Global Variables
**************************************************************************************************************************/
float g_fPCLK = 225000000.0;  //PCLK = 225 MHz default for ADSP-246x

/*************************************************************************************************************************
									Function Definitions
**************************************************************************************************************************/

/**************************************************************************************************************************
	Function name	:   Init_PLL
    Description		:   This function initializes the PLL for CCLK = 450 MHz and SCLK/PCLK = 225 MHz
	Arguments		:	None
	Return value	:   None
**************************************************************************************************************************/
void Init_PLL(void)
{
	 int iTemp,i;
	 

	#if defined(__214xx__)
    
		 //Step 1 - change the PLLD to 4
		 iTemp=*pPMCTL;
		 iTemp&=~PLLD16 ;
		 iTemp|=(PLLD4 | DIVEN) ;
		 *pPMCTL = iTemp;

		 //Step 2 - wait for dividers to stabilize
		 for(i=0;i<16;i++);

		 //Step 3 - set INDIV bit now to bring down the VCO speed and enter the bypass mode
		 iTemp&=~DIVEN;
		 iTemp|=(INDIV | PLLBP);
		 *pPMCTL = iTemp;

		 //Step 4 - wait for the PLL to lock
		 for(i=0;i<4096;i++)

		 //Step 5 - come out of the bypass mode
		 iTemp=*pPMCTL;
		 iTemp&=~PLLBP;
		 *pPMCTL = iTemp;

		 //Step 6 - wait for dividers to stabilize
		 for(i=0;i<16;i++);
		 
	#endif
	
		 //Step 7 - set the required PLLM and INDIV values here  and enter the bypass mode
		
		 iTemp = *pPMCTL;
		 iTemp&=~ (INDIV | PLLM63);
		
		 #if defined(__2146x__)
		 		//PLLM=18, INDIV=0,  fVCO=2*PLLM*CLKIN = 2*18*25 = 900 MHz
		 		iTemp|= (PLLM18| PLLBP);
		 		
		 #elif defined(__2147x__) || defined(__2148x__)
		 		//PLLM=18, INDIV=0,  fVCO=2*PLLM*CLKIN = 2*16*25 = 800 MHz
		 		iTemp|= (PLLM16| PLLBP);
		 		
		 #elif defined(__2137x__) 
		 		//PLLM=16, INDIV=0,  fVCO=2*PLLM*CLKIN = 2*16*16.384 = 524.288 MHz
		 		iTemp|= (PLLM16| PLLBP);
		 	
 		 #elif defined(__2136x__) 	
 		 		//PLLM=27, INDIV=1,  fVCO=2*PLLM*CLKIN/2 = 2*16*24.576/2 = 663.552 MHz
		 		iTemp|= (PLLM16| PLLBP);
		 		
		 #elif defined(__2126x__) 	
 		 		//PLLM=16, INDIV=0,  fVCO=2*PLLM*CLKIN = 2*16*25 = 800 MHz
		 		iTemp|= (PLLM16| PLLBP);
		 		
		 #endif		 		
		 
		 *pPMCTL = iTemp;

		 //Step 8 - wait for the PLL to lock
		 for(i=0;i<4096;i++);

		 //Step 9 - come out of the bypass mode
		 iTemp = *pPMCTL;
		 iTemp&=~PLLBP;
		 *pPMCTL=iTemp;

		 //Step 10 - wait for dividers to stabilize
		 for(i=0;i<16;i++);

		 //Step 11 - set the required values of PLLD(=2), LPCKR and DDR2CKR (=2) here
		
		 
	#if defined(__2146x__)
	 	 // fCCLK = fVCO/PLLD = 900/2 = 450 MHz, fDDR2CLK = fCCLK/DDR2CKR = 450/2 = 225 MHz, fLPCLK = 450/3 = 150 MHz, fPCLK = 450/2 = 225 MHz
		 iTemp=*pPMCTL;
		 iTemp&=~(PLLD16 | 0x1C0000 | LPCKR4 );
		 iTemp|= (DDR2CKR2 | LPCKR3 | PLLD2 | DIVEN);
		 *pPMCTL=iTemp;
		 
		 g_fPCLK=225000000.0;
		 
	#elif defined(__2147x__) || defined(__2148x__)
		// fCCLK = fVCO/PLLD = 800/2 = 400 MHz, fSDCLK = fCCLK/SDCKR = 400/2.5 = 160 MHz, fPCLK = 400/2 = 200 MHz
		 iTemp=*pPMCTL;
		 iTemp&=~(PLLD16 | 0x1C0000 );
		 iTemp|= (SDCKR2_5 | PLLD2 | DIVEN);
		 *pPMCTL=iTemp;
		 
		 g_fPCLK=200000000.0;
		 
	#elif defined(__2137x__) 
		 // fCCLK = fVCO/PLLD = 524.288/2 = 262.144 MHz, fSDCLK = fCCLK/SDCKR = 262.144/2 = 131.072 MHz, fPCLK = 262.144/2 = 131.072 MHz
	 	 iTemp=*pPMCTL;
		 iTemp&=~(PLLD8 | 0x1C0000 );
		 iTemp|= (SDCKR2 | PLLD1 | DIVEN);
		 *pPMCTL=iTemp;
		 
		 g_fPCLK=131072000.0;
	
			 
	#elif defined(__ADSP21367__) || defined(__ADSP21368__) || \
    	defined(__ADSP21369__)
    
    	 // fCCLK = fVCO/PLLD = 663.552/2 = 331.776 MHz, fSDCLK = fCCLK/SDCKR = 331.776/2 = 165.888 MHz, fPCLK = 331.776/2 = 165.888 MHz
	 	 iTemp=*pPMCTL;
		 iTemp&=~(PLLD8 | 0x1C0000 );
		 iTemp|= (SDCKR2 | PLLD1 | DIVEN);
		 *pPMCTL=iTemp;
		 
		 g_fPCLK=165888000.0;
	
		 	
	#elif defined(__ADSP21362__) || defined(__ADSP21363__) || defined(__ADSP21364__) || defined(__ADSP21365__) || defined(__ADSP21366__)
		 // fCCLK = fVCO/PLLD = 663.552/2 = 331.776 MHz, fSDCLK = fCCLK/SDCKR = 331.776/2 = 165.888 MHz, fPCLK = 331.776/2 = 165.888 MHz
	 	 iTemp=*pPMCTL;
		 iTemp&=~(PLLD8 | 0x1C0000 );
		 iTemp|= (PLLD1 | DIVEN);
		 *pPMCTL=iTemp;
		 
		 g_fPCLK=165888000.0;
		 
	 #elif defined(__2126x__) 	
	 	 // fCCLK = fVCO/2*PLLD = 800/4 = 200 MHz, fPCLK = fCCLK = 200 MHz
	 	 iTemp=*pPMCTL;
		 iTemp&=~(PLLD8 | 0x1C0000 );
		 iTemp|= (PLLD2 | DIVEN);
		 *pPMCTL=iTemp;
		 
		 g_fPCLK=200000000.0;
		 	 
	#endif
		 //Step 12 - wait for the dividers to stabilize
		 for(i=0;i<16;i++);
		 

		
}




#if defined(__ADSP21367__) || defined(__ADSP21368__) || \
    	defined(__ADSP21369__) || defined(__2137x__) || defined(__214xx__)
    
/**************************************************************************************************************************
	Function name	:   Init_SRU
    Description		:   This function initializes the DRAM (SDRAM/DDR2) controller
	Arguments		:	None
	Return value	:   None
**************************************************************************************************************************/
void Init_DRAM(void)
{
	
	int iTemp, i;


	#if defined(__2146x__)
	
		//DDR2 controller initialization for 2146x processors
		
		*pDLL0CTL1 |= 0xE00;
		*pDLL1CTL1 |= 0xE00;

	    for (i=0; i<8; i++)
	          NOP;
	    for (i=0; i<dll_lock_period; i++)
	          NOP;

		*pEPCTL= 0x0;
		*pEPCTL = B0SD | EPBRROT | DMAPRROT | NOFRZDMA | NOFRZCR | NOFRZSP ;

		*pDDR2RRC = DDR2TRFC29 | RDIV;
		*pDDR2CTL1 = DDR2TFAW12 | DDR2TRRD3 | DDR2TRTP2 | DDR2TRCD4 | DDR2TWTR2 | DDR2TRP4 | DDR2TRAS10 ;
		*pDDR2CTL2 = DDR2MR | DDR2TWR4 | DDR2CAS4 | DDR2BL4 ;
		*pDDR2CTL3 = DDR2EXTMR1 | DDR2AL0 | DDR2ODTDIS;
		*pDDR2CTL4 = DDR2EXTMR2;
		*pDDR2CTL5 = DDR2EXTMR3;
		*pDDR2CTL0 = DDR2PSS | DDR2RAW13 | DDR2CAW10 | DDR2BC8 | DDR2WDTHx16 | DDR2OPT | DDR2MODIFY1;

		while(!(*pDDR2STAT0 & DDR2DLLCALDONE))
			NOP;
			NOP;
			NOP;
			NOP;
			NOP;	

	#else

		//SDRAM controller initialization for other processors
		
		//Enable MS2 form the mux
		#if defined(__ADSP21367__) || defined(__ADSP21368__) || defined(__ADSP21369__)
		
			*pSYSCTL|=MSEN;
			
		#endif
		
			*pEPCTL&=~(B0SD|B1SD|B2SD|B3SD);
			*pEPCTL|=EPCTL_VALUE;
		
			*pSDRRC = SDRRC_VALUE;
			
			*pSDCTL = SDCTL_VALUE;
			
				
			//Dummy read to initiate power up sequencing		
			#if defined(__ADSP21367__) || defined(__ADSP21368__) || defined(__ADSP21369__)
		
				iTemp=*((volatile int*)0x8000000);
				
			#else
			
				iTemp=*((volatile int*)0x200000);
			
			#endif
			
	#endif

	
}


/**************************************************************************************************************************
	Function name	:   Init_SRU
    Description		:   This function initializes the SRU
	Arguments		:	None
	Return value	:   None
**************************************************************************************************************************/
void Init_SRU(void)
{

    SRU(SPI_MOSI_O,DPI_PB01_I);      //Connect MOSI to DPI PB1.
    SRU(DPI_PB02_O, SPI_MISO_I);     //Connect DPI PB2 to MISO.
    SRU(SPI_CLK_O, DPI_PB03_I);      //Connect SPI CLK to DPI PB3.
   // SRU(SPI_FLG0_O, DPI_PB05_I);     //Connect SPI FLAG3 to DPI PB5.
   
    SRU(HIGH, DPI_PB05_I);     //Drive the CS high initially
    
    SRU(HIGH, DPI_PBEN01_I);
    SRU(LOW, DPI_PBEN02_I);
    SRU(SPI_CLK_PBEN_O, DPI_PBEN03_I);
    SRU(HIGH, DPI_PBEN05_I);	
}

#endif

/**************************************************************************************************************************
	Function name	:   Setup_System_For_FLASH
    Description		:   This function initializes the system (PLL, SRU, SPI baud rate etc) for the FLASH
	Arguments		:	Parameter			 |	Description					| Valid values
						fSPICLK				 |  SPICLK speed 			    | Float
	Return value	:   None
**************************************************************************************************************************/
void Setup_System_For_FLASH(float fSPICLK)
{
	//SRU Initilzation required only for processors 21367/8/9 onwards
	#if defined(__ADSP21367__) || defined(__ADSP21368__) || \
    	defined(__ADSP21369__) || defined(__2137x__) || defined(__214xx__)
    
		Init_SRU();
		
	#else	
		//Set FLAG0 as output
		asm("bit set FLAGS 0x3;");
		
	#endif
	
	SPI_Set_Baud_Rate(fSPICLK, g_fPCLK);
}

